local addonInfo, privateVars = ...

---------- init namespace ---------

if not EnKai then EnKai = {} end

if not privateVars.uiFunctions then privateVars.uiFunctions = {} end

local uiFunctions   = privateVars.uiFunctions
local internal      = privateVars.internal
local data          = privateVars.data
local mapData       = privateVars.mapData

---------- addon internal function block ---------

local function _uiMap(name, parent)

	if EnKai.internal.checkEvents (name, true) == false then return nil end 

  ---------- VARIABLES ---------- 

  local activeMap = nil;
  local activeType = nil;
  local mapInfo = nil
  local scale = nil
  local scaleStep = nil
  local x, y
  local drag = false
  local mouseData = nil
  local coordX, coordY
  local elements = {}
  local maximized = false
  local maximizedX, maximizedY = 1, 1
  local maximizedScale = 1
  local width, height = 425, 370
  local maximizedWidth, maximizedHeight = 800, 600
  local origCoordX, origCoordY = nil, nil, nil
  local origX, origY = nil, nil
  local maxZoom = 4
  local cursorX, cursorY
  local coordsArea = {}
  local waypoint = nil
  local cursorCoordX, cursorCoordY
  local animated = true
  local smoothScroll = true  
  
  ---------- UI ELEMENTS ----------

  local ui = EnKai.uiCreateFrame("nkWindowElement", name .. ".window", parent)
  ui:SetWidth(425)
  ui:SetHeight(370)
  ui:SetDragable(true)
  ui:SetCloseable(false)
  ui:SetFontSize(12)
  
  local iconZoomIn = EnKai.uiCreateFrame("nkTexture", name ..".iconZoomIn", ui:GetHeader())
  iconZoomIn:SetTextureAsync("EnKai", "gfx/iconZoomIn.png")
  iconZoomIn:SetPoint("CENTERRIGHT", ui:GetHeader(), "CENTERRIGHT", -5, 0)
  iconZoomIn:SetHeight(16)
  iconZoomIn:SetWidth(16)
  
  local iconZoomOut = EnKai.uiCreateFrame("nkTexture", name ..".iconZoomOut", ui:GetHeader())
  iconZoomOut:SetTextureAsync("EnKai", "gfx/iconZoomOut.png")
  iconZoomOut:SetPoint("CENTERRIGHT", iconZoomIn, "CENTERLEFT", -2, 0)
  iconZoomOut:SetHeight(16)
  iconZoomOut:SetWidth(16)
  
  local iconMinMax = EnKai.uiCreateFrame("nkTexture", name ..".iconMinMax", ui:GetHeader())
  iconMinMax:SetTextureAsync("EnKai", "gfx/iconMaximize.png")
  iconMinMax:SetPoint("CENTERRIGHT", iconZoomOut, "CENTERLEFT", -2, 0)
  iconMinMax:SetHeight(16)
  iconMinMax:SetWidth(16)
  
  local coordLabel = EnKai.uiCreateFrame("nkText", name .. ".coordLabel", ui:GetHeader())
  coordLabel:SetFontSize(12)
  coordLabel:SetFontColor(1, 1, 1, 1)
  coordLabel:SetPoint("CENTER", ui:GetHeader(), "CENTER") 
  
  local mask = UI.CreateFrame('Mask', name, ui:GetContent())
  mask:SetPoint("TOPLEFT", ui:GetContent(), "TOPLEFT")
  mask:SetPoint("BOTTOMRIGHT", ui:GetContent(), "BOTTOMRIGHT")
  
  local map = EnKai.uiCreateFrame("nkTexture", name .. ".map", mask)
  map:SetLayer(1)
  
  local tooltip = EnKai.uiCreateFrame("nkTooltip", name .. ".tooltip", ui)
  tooltip:SetVisible(false)
  tooltip:SetLayer(999)
  
  ---------- LOCAL METHODS ----------
 
  local function _fctRedraw ()
  
    local debugId  
    if nkDebug then debugId = nkDebug.traceStart (Inspect.Addon.Current(), "EnKai _uiMap:Redraw") end    
  
    local thisScale = scale
    if maximized == true then thisScale = maximizedScale end
    local beforeScale = thisScale
    
    --print (mapInfo.width * thisScale, mask:GetWidth(), mapInfo.height * thisScale, mask:GetHeight())
    
    if mapInfo.width * thisScale < mask:GetWidth() or mapInfo.height * thisScale < mask:GetHeight() then
    
      local xScale, yScale = 0, 0
    
      xScale = 1 / mapInfo.width * mask:GetWidth()
      yScale = 1 / mapInfo.height * mask:GetHeight()
      
      --print (xScale, yScale)
      
      if xScale > yScale then
        thisScale = xScale
      else
        thisScale = yScale
      end
    
    end
    
    map:SetWidth(mapInfo.width * thisScale)
    map:SetHeight(mapInfo.height * thisScale)
    
    --print ('redraw - setCoord')
    
    ui:SetCoord()
    
    --print (thisScale)
    
    for key, thisElement in pairs (elements) do
      thisElement:SetZoom(thisScale)
      thisElement:SetCoord()
      
      local eleX, eleY = thisElement:GetCoord()
      
      if eleX < coordsArea.x1 or eleX > coordsArea.x2 or eleY < coordsArea.y1 or eleY > coordsArea.y2 then
        thisElement:SetVisible(false)
      elseif thisElement:GetWidth() <= 10 and thisElement:GetHeight() <= 10 then
        thisElement:SetVisible(false) 
      else
        thisElement:SetVisible(true)
      end
      
    end
    
    if beforeScale ~= thisScale then
      if maximized == true then maximizedScale = thisScale else scale = thisScale end
      EnKai.eventHandlers[name]["Zoomed"](thisScale, maximized)
    end
    
    if nkDebug then nkDebug.traceEnd (Inspect.Addon.Current(), "EnKai _uiMap:Redraw", debugId) end
    
  end
 
  local function _fctZoomOut()

    if map:GetWidth() >= mask:GetWidth() and map:GetHeight() >= mask:GetHeight() then
    
      local debugId  
      if nkDebug then debugId = nkDebug.traceStart (Inspect.Addon.Current(), "EnKai _uiMap:ZoomOut") end
    
      local thisScale = scale
      if maximized == true then thisScale = maximizedScale end
    
      if (thisScale - scaleStep >= 0) then
        if maximized == true then 
          maximizedScale = maximizedScale - scaleStep
          thisScale = maximizedScale
        else
          scale = scale - scaleStep
          thisScale = scale
        end        
        _fctRedraw()
        EnKai.eventHandlers[name]["Zoomed"](thisScale, maximized)
      end 
      
      if nkDebug then nkDebug.traceEnd (Inspect.Addon.Current(), "EnKai _uiMap:ZoomOut", debugId) end     
    end
  end
  
  local function _fctZoomIn()
    
    local debugId
    if nkDebug then debugId = nkDebug.traceStart (Inspect.Addon.Current(), "EnKai _uiMap:ZoomIn") end
  
    local thisScale = scale
    if maximized == true then thisScale = maximizedScale end
    
    if thisScale < maxZoom then
      if maximized == true then 
        maximizedScale = maximizedScale + scaleStep
      else
        scale = scale + scaleStep
      end        
        
      _fctRedraw()
      EnKai.eventHandlers[name]["Zoomed"](thisScale, maximized)
    end
    
    if nkDebug then nkDebug.traceEnd (Inspect.Addon.Current(), "EnKai _uiMap:ZoomIn", debugId) end
    
  end
  
  local function _fctUpdateCoord(cursorX, cursorY)
    
    --print (cursorX, mask:GetLeft(), map:GetLeft())
    
    local diffX = mask:GetLeft() - map:GetLeft() + (cursorX - mask:GetLeft())
    local diffY = mask:GetTop() - map:GetTop() + (cursorY - mask:GetTop())
    
    local xP = 1 / map:GetWidth() * diffX
    local yP = 1 / map:GetHeight() * diffY
    
    cursorCoordX = math.floor(((mapInfo.x2 - mapInfo.x1) * xP) + mapInfo.x1)
    cursorCoordY = math.floor(((mapInfo.y2 - mapInfo.y1) * yP) + mapInfo.y1) 
    
    coordLabel:SetText(string.format("%d / %d", cursorCoordX, cursorCoordY))
    
  end
  
  local function _fctPosition(newX, newY)
    
    x, y = newX, newY
    
    if x + map:GetWidth() < mask:GetWidth() then
       x = mask:GetWidth() - map:GetWidth()
    elseif x > 0 then x = 0 end
    
    if y + map:GetHeight() < mask:GetHeight() then
       y = mask:GetHeight() - map:GetHeight()
    elseif y > 0 then y = 0 end

    map:SetPoint("TOPLEFT", mask, "TOPLEFT", x, y)
        
  end
  
  local function _fctProcessWayPoint () 
  
    if waypoint ~= nil then
      if cursorCoordX == waypoint.x and cursorCoordY == waypoint.y then
        waypoint = nil
        Command.Map.Waypoint.Clear()    
      else
        Command.Map.Waypoint.Clear()
        Command.Map.Waypoint.Set (cursorCoordX, cursorCoordY)
        waypoint = {x = cursorCoordX, y = cursorCoordY}
      end
    else
      waypoint = {x = cursorCoordX, y = cursorCoordY}
      Command.Map.Waypoint.Set (cursorCoordX, cursorCoordY)
    end
  
  end
 
  ---------- PUBLIC METHODS ----------
 
  function ui:ToggleMinMax(internal)
  
    if maximized == true then
      maximized = false
      iconMinMax:SetTextureAsync("EnKai", "gfx/iconMaximize.png")
      
      maximizedWidth = ui:GetWidth()
      maximizedHeight = ui:GetHeight()
      
      coordX, coordY = origCoordX, origCoordY
            
      ui:SetWidth(width)
      ui:SetHeight(height)
      ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT", origX, origY)
    else
      maximized = true
      iconMinMax:SetTextureAsync("EnKai", "gfx/iconMinimize.png")
      
      width = ui:GetWidth()
      height = ui:GetHeight()
      
      origCoordX, origCoordY =  coordX, coordY
      origX, origY = ui:GetLeft(), ui:GetTop()
      
      ui:SetWidth(maximizedWidth)
      ui:SetHeight(maximizedHeight)
      ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT", maximizedX, maximizedY)      
    end
    
    _fctRedraw()
    
    if internal == true then EnKai.eventHandlers[name]["Toggled"]() end
  end
  
  function ui:SetMap(activeType, mapName)
  
    if activeMap == mapName then return end
  
    activeType = activeType
    activeMap = mapName
    
    mapInfo = internal.getMapData (mapName) 
    
    if mapInfo.width <= mapInfo.height then
      scaleStep = 1 / mapInfo.width * mask:GetWidth()
    else
      scaleStep = 1 / mapInfo.height * mask:GetHeight()
    end 
    
    if scale == nil then scale = scaleStep end
    
    local addon = mapInfo.addon
    if addon == nil then addon = "Rift" end
    map:SetTextureAsync(addon, mapInfo.path)
    
    x, y = nil, nil
    
    _fctRedraw()
    
  end
  
  function ui:SetZoom (newZoomLevel, thisMaximized)
    
    if newZoomLevel <= 0 or newZoomLevel > maxZoom then return end
    
    for idx = 0, maxZoom, scaleStep do
      if newZoomLevel < idx then
        if thisMaximized == true then
          maximizedScale = newZoomLevel - scaleStep
          if maximized == true then _fctRedraw () end
        else
          scale = newZoomLevel - scaleStep
          if maximized == false then _fctRedraw () end
        end
                
        return
      end
    end
    
  end
  
  function ui:SetCoord (newCoordX, newCoordY)
    
    if coordX == newCoordX and coordY == newCoordY then return end
    
    if newCoordX ~= nil then coordX = newCoordX end
    if newCoordY ~= nil then coordY = newCoordY end
    
    if coordX == nil then coordX = (mapInfo.x2 - mapInfo.x1) / 2 end
    if coordY == nil then coordY = (mapInfo.y2 - mapInfo.y1) / 2 end
    
    coordLabel:SetText(string.format("%d / %d", coordX, coordY))

    local pX = 1 / (mapInfo.x2 - mapInfo.x1) * (coordX - mapInfo.x1)
    local pY = 1 / (mapInfo.y2 - mapInfo.y1) * (coordY - mapInfo.y1)
        
    local newX = (mask:GetWidth() / 2) - (map:GetWidth() * pX)
    local newY = (mask:GetHeight() / 2) - (map:GetHeight() * pY)
    
    if smoothScroll == false then newX, newY = math.floor(newX), math.floor(newY) end
    
    if newX == x and newY == y then return end
    
    _fctPosition(newX, newY)

    local xP1 = 1 / map:GetWidth() * (mask:GetLeft() - map:GetLeft())
    local yP1 = 1 / map:GetHeight() * (mask:GetTop() - map:GetTop())
    
    local xP2 = 1 / map:GetWidth() * (mask:GetLeft() - map:GetLeft() + mask:GetWidth())
    local yP2 = 1 / map:GetHeight() * (mask:GetTop() - map:GetTop() + mask:GetHeight())
    
    coordsArea = {x1 = ((mapInfo.x2 - mapInfo.x1) * xP1) + mapInfo.x1,
                  x2 = ((mapInfo.x2 - mapInfo.x1) * xP2) + mapInfo.x1,
                  y1 = ((mapInfo.y2 - mapInfo.y1) * yP1) + mapInfo.y1, 
                  y2 = ((mapInfo.y2 - mapInfo.y1) * yP2) + mapInfo.y1}
    
    for id, element in pairs(elements) do
      
      local eleX, eleZ = element:GetCoord()
      
      if eleX >= coordsArea.x1 and eleX <= coordsArea.x2 and eleZ >= coordsArea.y1 and eleZ <= coordsArea.y2 then
        element:SetVisible(true)
      else
        element:SetVisible(false)
      end
      
    end 
    
  end
  
  function ui:SetPointMaximized(x, y)
    maximizedX = x
    maximizedY = y
  end
  
  function ui:SetWidthMaximized(newWidth)
    maximizedWidth = newWidth
  end
  
  function ui:SetHeightMaximized(newHeight)
    maximizedHeight = newHeight
  end
    
  function ui:AddElement (newElement)
  
    -- check einbauen ob an exakt der gleichen Stelle bereits exakt das gleiche Element besteht
    
    local debugId  
    if nkDebug then debugId = nkDebug.traceStart (Inspect.Addon.Current(), "EnKai _uiMap:AddElement") end
  
    if mapData.mapElements[newElement.type] == nil then
      if nkDebug then print (newElement.type) end 
      return 
    end
    if elements[newElement.id] ~= nil then return end
    
    local thisElement
    local mapInfo = mapData.mapElements[newElement.type]
    
    if mapInfo.anim ~= nil then
      thisElement = EnKai.uiCreateFrame("nkMapElementCanvas", newElement.type .. "." .. EnKai.tools.uuid(), mask)
    elseif mapInfo.gfxType == nil or string.lower(mapInfo.gfxType) == 'texture' then
      thisElement = EnKai.uiCreateFrame("nkMapElementTexture", newElement.type .. "." .. EnKai.tools.uuid(), mask)
    elseif string.lower(mapInfo.gfxType) == "canvas" then
      thisElement = EnKai.uiCreateFrame("nkMapElementCanvas", newElement.type .. "." .. EnKai.tools.uuid(), mask)
    end
    
    local thisScale = scale
    if maximized == true then thisScale = maximizedScale end
    
    thisElement:SetParentMap(ui)    
    
    if newElement.radius ~= nil then thisElement:SetRadius(newElement.radius) end
    thisElement:SetType(newElement.type)
    
    --if (newElement.type == "QUEST.START") then print(newElement.title); dump(newElement.descList) end
    thisElement:SetToolTip(newElement.title, newElement.descList)
    
    if newElement.angle ~= nil and thisElement.SetAngle ~= nil then thisElement:SetAngle(newElement.angle) end    
    
    thisElement:SetZoom(thisScale, maximized)
    
    local thisY = newElement.coordY
    if newElement.coordZ ~= nil then thisY = newElement.coordZ end
    
    if (thisY == nil or newElement.coordX == nil) then
      dump (newElement)
    else
      thisElement:SetZoom(thisScale, maximized)
      thisElement:SetCoord(newElement.coordX, thisY)
    end
    
    thisElement:SetVisible(true)
               
    elements[newElement.id] = thisElement
    
    if nkDebug then nkDebug.traceEnd (Inspect.Addon.Current(), "EnKai _uiMap:AddElement", debugId) end
    
  end
  
  function ui:ChangeElement (updateElement)
  
    if elements[updateElement.id] == nil then return end
    
    local debugId  
    if nkDebug then debugId = nkDebug.traceStart (Inspect.Addon.Current(), "EnKai _uiMap:ChangeElement") end
    
    local thisElement = elements[updateElement.id]
    
    local thisY = updateElement.coordY
    if updateElement.coordZ ~= nil then thisY = updateElement.coordZ end
    
    thisElement:SetCoord(updateElement.coordX, thisY)
    
    if updateElement.angle ~= nil and thisElement.SetAngle ~= nil then thisElement:SetAngle(updateElement.angle) end
    
    if nkDebug then nkDebug.traceEnd (Inspect.Addon.Current(), "EnKai _uiMap:ChangeElement", debugId) end
    
  end
  
  function ui:RemoveElement(removeElement)
  
    if elements[removeElement] == nil then return end
    
    local debugId  
    if nkDebug then debugId = nkDebug.traceStart (Inspect.Addon.Current(), "EnKai _uiMap:RemoveElement") end
    
    local thisElement = elements[removeElement]
    
    if thisElement:GetTooltip() == true then tooltip:SetVisible(false) end
    
    thisElement:destroy()
    elements[removeElement] = nil
  
    if nkDebug then nkDebug.traceEnd (Inspect.Addon.Current(), "EnKai _uiMap:RemoveElement", debugId) end
  
  end
  
  function ui:GetScale()
    if maximized == true then
      return maximizedScale, true
    else
      return scale, false
    end
  end
  
  function ui:UpdateMapInfo(newMapInfo)
    mapInfo = newMapInfo
    _fctRedraw ()
  end
  
  function ui:SetAnimated(flag) 
    animated = flag
    
    for key, element in pairs(elements) do
      if element.SetAnimated ~= nil then element:SetAnimated(flag) end 
    end
     
  end
  
  function ui:GetAnimated() return animated end
  function ui:GetMapInfo() return mapInfo end
  function ui:GetMap() return map end
  function ui:GetMapName() return activeMap end
  function ui:GetMask() return mask end
  function ui:GetTooltip() return tooltip end
  function ui:GetCoords() return coordX, coordY end
  function ui:GetElement(key) return elements[key] end
  function ui:SetSmoothScroll(flag) smoothScroll = flag end
  
  ---------- EVENTS ---------- 
   
  Command.Event.Attach(EnKai.events[name .. '.window'].Moved, function (_, newX, newY)
    if maximized == true then
      maximizedX, maximizedY = newX, newY
    else
      origX, origY = newX, newY
    end
    EnKai.eventHandlers[name]["Moved"](newX, newY, maximized)
  end, name .. '.window.Moved')
  
  Command.Event.Attach(EnKai.events[name .. '.window'].Resized, function (_, newWidth, newHeight)
    
    _fctRedraw()
    _fctPosition(x, y)
    
    if maximized == true then
      maximizedHeight = newHeight
      maximizedWidth = newWidth
    else
      width = newWidth
      height = newHeight
    end
    
    EnKai.eventHandlers[name]["Resized"](newWidth, newHeight, maximized)
  end, name .. '.window.Resized')
  
  ui:GetContent():EventAttach(Event.UI.Input.Mouse.Wheel.Back, function () _fctZoomOut() end, ui:GetName() .. ".Mouse.Wheel.Back")
  
  ui:GetContent():EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function () _fctZoomIn() end, ui:GetName() .. ".Mouse.Wheel.Forward")
  
  ui:GetContent():EventAttach(Event.UI.Input.Mouse.Left.Down.Bubble, function ()
    
    drag = true
    mouseData = Inspect.Mouse()
    
  end, ui:GetName() .. ".Mouse.Left.Down.Bubble")
  
  ui:GetContent():EventAttach(Event.UI.Input.Mouse.Cursor.Move.Bubble, function (self, _, posX, posY)
  
    if drag ~= true then
      _fctUpdateCoord(posX, posY) 
      return 
    end
    
    local diffX, diffY = posX - mouseData.x, posY - mouseData.y
    
    local xP = 1 / map:GetWidth() * math.abs(diffX)
    local yP = 1 / map:GetHeight() * math.abs(diffY)
        
    if diffX < 0 then
      coordX = coordX + ((mapInfo.x2 - mapInfo.x1) * xP)
    else
      coordX = coordX - ((mapInfo.x2 - mapInfo.x1) * xP)       
    end
    
    if diffY < 0 then
      coordY = coordY + ((mapInfo.y2 - mapInfo.y1) * yP)
    else
      coordY = coordY - ((mapInfo.y2 - mapInfo.y1) * yP)       
    end
    
    ui:SetCoord ()
    mouseData = Inspect.Mouse()
        
  end, ui:GetName() .. ".Cursor.Move.Bubble")
 
  ui:GetContent():EventAttach(Event.UI.Input.Mouse.Left.Up.Bubble, function ()
    drag = false
  end, ui:GetName() .. ".Mouse.Left.Up.Bubble")
  
  ui:GetContent():EventAttach(Event.UI.Input.Mouse.Left.Upoutside, function ()
    drag = false
  end, ui:GetName()  .. ".Mouse.left.Upoutside")
  
  ui:GetContent():EventAttach(Event.UI.Input.Mouse.Right.Down.Bubble, function ()
    
    _fctProcessWayPoint()

  end, ui:GetName() .. ".Mouse.Right.Down.Bubble")  
  
  iconZoomIn:EventAttach(Event.UI.Input.Mouse.Left.Down, function () _fctZoomIn() end, iconZoomIn:GetName() .. ".iconZoomIn.Mouse.Left.Down")
  iconZoomOut:EventAttach(Event.UI.Input.Mouse.Left.Down, function () _fctZoomOut() end, iconZoomOut:GetName() .. ".iconZoomOut.Mouse.Left.Down")
  iconMinMax:EventAttach(Event.UI.Input.Mouse.Left.Down, function () ui:ToggleMinMax(true) end, iconMinMax:GetName() .. ".iconMinMax.Mouse.Left.Down")
  
  ---------- EVENT HANDLERS ---------- 
  
  EnKai.eventHandlers[name]["Moved"], EnKai.events[name]["Moved"] = Utility.Event.Create(addonInfo.identifier, name .. "Moved")
  EnKai.eventHandlers[name]["Resized"], EnKai.events[name]["Resized"] = Utility.Event.Create(addonInfo.identifier, name .. "Resized")
  EnKai.eventHandlers[name]["Zoomed"], EnKai.events[name]["Zoomed"] = Utility.Event.Create(addonInfo.identifier, name .. "Zoomed")
  EnKai.eventHandlers[name]["Toggled"], EnKai.events[name]["Toggled"] = Utility.Event.Create(addonInfo.identifier, name .. "Toggled")
  
  return ui
	
end

uiFunctions.NKMAP = _uiMap