local addonInfo, internal = ...

---------- init namespace ---------

if not nkRadial then nkRadial = {} else return end

internal.fct        = {}
internal.uiElements = {}
internal.lang       = {}
internal.data       = {}

local intFct     = internal.fct
local uiElements = internal.uiElements
local data       = internal.data

---------- init local variables ---------

local _preDefinedSlots = {
  potion    = { itemcategories = {"consumable potion" }}, 
  teleport  = { abilityids = {"A473DB77B2A283628", "A665FDAC7EDD37636", "A3C5AEC64D3793518", "A313102E6A9A460E8", "A6B16924B96299E96", "A768D08F826CDC3DB", "A5A5C6D30430A9E3E", "A677B0D96691D0433", "A657AA5389319EF0F"}, itemtypes = {"IFAFB17B78F862295,2372E487E1EDD0FA,,,,,,", "I139E7010C5722EA5,DF73A68C936CC83F,,,,,,", "I1C6AA9C8821E8F6D,22FC684EB50BD66F,,,,,,", "I731C1B4F46334030,293CCC1E8957C704,,,,,,"}}, 
  guild     = { abilityids = {"A5F77DAA65FAED9E3"}, itemtypes = {"I1F832DB376787EBF,D912113BAA7304B3,,,,,,"} }, 
  food      = { itemcategories = {"consumable food" }}, 
  buff      = { abilityids = {"*"}},
  recycle   = { abilityids = {"A0CFDEF4294B17BA3", "A2A25DCB2144D5993", "A0000000A9C3986D3", "A7EBDFE5C5F1E8E21", "A7D4DFB3F202C268A", "A559893A4522C10A9"}},
  utility   = { abilityids = {"A0AE89C20B7D46D0F", "A3416A1A673AEA8DF", "A24FE01816BA3C9E7", "A38DFCB8220EE4A2D", "A60203E8E82FC1D26", "A0A65783568220667", "A251F2E341BED68A3"}},
  patron    = { abilityids = {"A3D21C84687199B44", "A2EA96AD566F13C62", "A439CC66597456366", "A72F1082A8F6DD4C5", "AFD83E703841348C0", "A3F5DD24F7FEF4C53", "A4090FAAA693ABEEC", "A0530A5F074940F78", "A3F5E814D94343B3C", "A54F1DA73C9D1D272", "A3C7F03BBA1BBE8AD", "A665FDDAC7EDD37636", "A1E12EB8E986727F7"}},
  various   = { abilityids = {"*"}, itemcategories = {"*"}}
}

---------- init variables ---------

data.buttonSubShown    = nil
data.maxButtons        = 0
data.stopAnimation     = false

data.slotIconDefaults  = { potion   = { type = "addon", path = "gfx/iconPotion.png"}, 
                           teleport = { type = "addon", path = "gfx/iconTeleport.png"},
                           guild    = { type = "addon", path = "gfx/iconBanner.png"},
                           food     = { type = "addon", path = "gfx/iconFood.png"},
                           buff     = { type = "addon", path = "gfx/iconBuff.png"},
                           utility  = { type = "addon", path = "gfx/iconCrafting.png"},
                           recycle  = { type = "addon", path = "gfx/iconRecycle.png"},
                           patron   = { type = "addon", path = "gfx/iconPatron.png"},
                           various  = { type = "addon", path = "gfx/iconVarious.png"}
                         }

---------- generate ui context ---------

uiElements.context = UI.CreateContext("nkRadial")
uiElements.context:SetStrata ('dialog')

uiElements.contextTop = UI.CreateContext("nkRadial.topmost")
uiElements.contextTop:SetStrata ('topmost')

uiElements.contextSecure = UI.CreateContext("nkRadial.Secure")
uiElements.contextSecure:SetSecureMode('restricted')

---------- make global functions local ---------

local oInspectItemDetail       = Inspect.Item.Detail
local oInspectAbilityNewDetail = Inspect.Ability.New.Detail
local oSGSub                   = string.gsub

---------- local function block ---------

local function _fctCooldown (_, info)

  local found = false
  
  for key, details in pairs (info) do
    
    found = false
    
    for idx = 1, #uiElements.buttons, 1 do
      for idx2 = 1, #uiElements.buttons[idx].sub, 1 do
        local type, id = uiElements.buttons[idx].sub[idx2]:GetType()
        if type == details.type and key == id then
          if details.remaining ~= nil then            
            if details.remaining > 3600 then
              uiElements.buttons[idx].sub[idx2]:SetCooldown(tostring(math.floor(details.remaining / 3600)).."h")
            elseif details.remaining > 60 then
              uiElements.buttons[idx].sub[idx2]:SetCooldown(tostring(math.floor(details.remaining / 60)).."m")
            else
              uiElements.buttons[idx].sub[idx2]:SetCooldown(tostring(math.floor(details.remaining)))
            end
          else
            uiElements.buttons[idx].sub[idx2]:SetCooldown(nil)  
          end
          
          found = true
          break
        end
        
        if found then break end
      end
    end
  end  

end

local function _fctCommandHandler (commandline) intFct.ShowConfig() end

local function _fctMain(_, addon)

  if addon == addonInfo.identifier then
    
    EnKai.cooldown.init()
    
    for slot, details in pairs(nkRadialSetup.slotSetup) do
      for idx = 1, #details, 1 do
        if details[idx].type ~= nil then
          EnKai.cooldown.subscribe(details[idx].type, details[idx].info)
        end
      end
    end
    
    Command.Event.Attach(EnKai.events["EnKai.CooldownManager"].start, _fctCooldown, "nkRadial.EnKai.CooldownManager.start")
    Command.Event.Attach(EnKai.events["EnKai.CooldownManager"].update, _fctCooldown, "nkRadial.EnKai.CooldownManager.update")
    Command.Event.Attach(EnKai.events["EnKai.CooldownManager"].stop, _fctCooldown, "nkRadial.EnKai.CooldownManager.stop")
  
    table.insert(Command.Slash.Register("nkRadial"), {_fctCommandHandler, "nkRadial", "config"})
    table.insert(Command.Slash.Register("nkRad"), {_fctCommandHandler, "nkRadial", "config"})
  
    intFct.buildButtons ()
    uiElements.tooltip = intFct.tooltip ()
    
    local items = { { label = internal.lang.buttonMenuConfig, callBack = function () intFct.ShowConfig() end} }

    if nkPanel ~= nil then
      uiElements.panel = intFct.nkPanelPlugin(items)
      nkPanel.api.registerPlugin('nkRadial', uiElements.panel)          
    else      
      EnKai.manager.init('nkRadial', items, nil)      
    end
    
    if (nkRadialSetup.tutorial == nil) then
      uiElements.tutorial = intFct.tutorial()
      nkRadialSetup.tutorial = true 
    end
    
    EnKai.version.init(addonInfo.toc.Identifier, addonInfo.toc.Version)
  end

end

local function _fctSettingsHandler(_, addon) 

  if addon == addonInfo.identifier then
  
    if nkRadialSetup == nil then
    
      nkRadialSetup = { locked = false,
                        x = 0, y = 0, startAngle = 0, mouseOverAlpha = 1, scale = 1, innerDesign = "default", outterDesign = "default", closingTime = 60, timedClose = true,
                        sections = { potion = true, teleport = true, guild = true, food = true, buff = true, utility = true, recycle = true, patron = false, various = false },
                        slotSetup = { potion = {},
                                      teleport =  {},
                                      guild =  {},
                                      food = {},
                                      buff = {},
                                      utility = {},
                                      recycle = {},
                                      patron =  {},
                                      various= {} 
                                    },
                        slotIcon = data.slotIconDefaults
                        }
                                    
      for k, v in pairs (_preDefinedSlots) do
        local temp = { {}, {}, {}, {}, {} }
        local count = 1
        
        if v.abilityids ~= nil then
          for idx = 1, #v.abilityids, 1 do
            if count <= 5 then
              local id = v.abilityids[idx]
              
              if id ~= "*" then            
                local err, details = pcall (oInspectAbilityNewDetail, id)
                if err == true and details ~= nil then          
                  temp[count] = { type = "ability", info = v.abilityids[idx]}
                  count = count + 1
                end
              end
            end
          end
        end
        
        if v.itemtypes ~= nil then      
          for idx = 1, #v.itemtypes, 1 do
           if count <= 5 then
              local id = v.itemtypes[idx]
              
              if id ~= "*" then                      
                
                local err, details = pcall (oInspectItemDetail, id)
                if err == true and details ~= nil then          
                  temp[count] = { type = "item", info = v.itemtypes[idx]}
                  count = count + 1
                end
              end
            end
          end
        end
        
        nkRadialSetup.slotSetup[k] = temp
      end
    end
  end
  
end

local function _fctHideMainButtons ()
  for idx = 1, #uiElements.buttons, 1 do
    uiElements.buttons[idx].main:SetVisible(false)
  end 
end

---------- addon internal function block ---------

function intFct.showButtons ()

  local baseRadius = uiElements.center:GetWidth() / 2
  
  local stepDegree = 360 / 8
  local maxDegree = (data.maxButtons-1) * stepDegree + 1
  
  local addCoRoutine = coroutine.create(
    function (buttons)
  
      for idx = 1, maxDegree, 10 do
        
        for idx2 = idx, idx + 9, 1 do
          
          for idx3 = 1, data.maxButtons, 1 do
            local max = (idx3 -1) * stepDegree + 1
                        
            if idx2 <= max then
              local pos = idx2 + nkRadialSetup.startAngle
              local thisPos = buttons[idx3].main:GetPosition()
              if pos >= thisPos then 
                buttons[idx3].main:Position (pos, baseRadius + buttons[idx3].main:GetWidth() / 2 )
              end
            end
          end
        end

        if data.stopAnimation == false then
          coroutine.yield(idx)
        else
          coroutine.yield(9999)
        end
      end
    end
  )
  
  for idx = 1, data.maxButtons, 1 do
    uiElements.buttons[idx].main:SetVisible(true)
  end 
  
  EnKai.coroutines.add ({ func = addCoRoutine, counter = maxDegree, active = true, para1 = uiElements.buttons })

end

function intFct.hideButtons ()

  local baseRadius = uiElements.center:GetWidth() / 2
  local stepDegree = 360 / 8
  local maxDegree = (data.maxButtons-1) * stepDegree + 1

  if data.buttonSubShown ~= nil then
    intFct.hideSubButtons(data.buttonSubShown)          
    data.buttonSubShown = nil
  end

  local addCoRoutine = coroutine.create(
    function (buttons)
  
      for idx = 1, maxDegree, 10 do
        
        for idx2 = idx, idx + 9, 1 do
          
          for idx3 = 1, #buttons, 1 do

            if Inspect.System.Secure() == true then coroutine.yield(9999) end

            local pos = maxDegree - idx2 + nkRadialSetup.startAngle + 1
            if pos < buttons[idx3].main:GetPosition() then
              local thisPos = buttons[idx3].main:GetPosition()
              if pos <= thisPos then
                buttons[idx3].main:Position (pos, baseRadius + buttons[idx3].main:GetWidth() / 2 )
              end
            end
          end
        end
        
        if data.stopAnimation == false then        
          coroutine.yield(idx)
        else
          coroutine.yield(9999)
        end
      end
    end)
    
  local callBack = function ()
    EnKai.events.addInsecure(function() _fctHideMainButtons() end, nil, nil)
  end
  
  EnKai.coroutines.add ({ func = addCoRoutine, counter = maxDegree, active = true, para1 = uiElements.buttons, callBack = callBack })

end

function intFct.showSubButtons (buttonNo, section)

  local baseRadius = uiElements.center:GetWidth() / 2
  
  local count = 0
  
  for idx = 1, 10, 1 do  
    if nkRadialSetup.slotSetup[section][idx] ~= nil and nkRadialSetup.slotSetup[section][idx].info ~= nil then
      count = idx
    end
  end     
  
  local mid = math.floor(count / 2)

  local addCoRoutine = coroutine.create(
    function (button)

      for idx = 1, 40 * nkRadialSetup.scale, 3 do
      
        local r = (baseRadius) + (button.sub[1]:GetWidth() / 2)  + 10 + idx
        local basePos = button.main:GetPosition ()
        
        for idx = 1, count, 1 do
          local diff = (idx - mid - 1) * 25
          if mid == count/2 then diff = diff + 12.5 end
          
          local thisRadius = button.sub[idx]:GetRadius()
          if r >= thisRadius then
            button.sub[idx]:Position(basePos + diff, r)
          end
        end 
        
        if data.stopAnimation == false then        
          coroutine.yield(idx)
        else
          coroutine.yield(9999)
        end
      end
    end)
  
  for idx = 1, #nkRadialSetup.slotSetup[section], 1 do
  
    local icon = nil
    local macro = ""
    local name = ""
    local desc = ""
    
    if nkRadialSetup.slotSetup[section][idx].type == "item" then
      local details = oInspectItemDetail(nkRadialSetup.slotSetup[section][idx].info)
      if details ~= nil then 
        icon = details.icon
        name = details.name
        macro = "use " .. oSGSub(details.name, "\n", "")
        desc = details.description
      end        
    elseif nkRadialSetup.slotSetup[section][idx].type == "ability" then
      local details = oInspectAbilityNewDetail(nkRadialSetup.slotSetup[section][idx].info)
      if details ~= nil then 
        icon = details.icon
        name = details.name
        desc = details.description
        macro = "cast " .. oSGSub(details.name, "\n", "") 
      end
    end 
    
    if icon ~= nil then
      uiElements.buttons[buttonNo].sub[idx]:SetTexture("Rift", icon)
      uiElements.buttons[buttonNo].sub[idx]:EventMacroSet(Event.UI.Input.Mouse.Left.Click, macro)
      uiElements.buttons[buttonNo].sub[idx]:SetVisible(true)
      uiElements.buttons[buttonNo].sub[idx]:SetTooltipInfo(name, desc)
      uiElements.buttons[buttonNo].sub[idx]:SetType(nkRadialSetup.slotSetup[section][idx].type, nkRadialSetup.slotSetup[section][idx].info)
    end
  end 
  
  EnKai.coroutines.add ({ func = addCoRoutine, counter = 40 * nkRadialSetup.scale, active = true, para1 = uiElements.buttons[buttonNo] })  

end

function intFct.hideSubButtons (buttonNo)

  local baseRadius = uiElements.center:GetWidth() / 2
  local section = uiElements.buttons[buttonNo].section
  
  local count = 0
  
  for idx = 1, 10, 1 do
    if nkRadialSetup.slotSetup[section][idx] ~= nil and nkRadialSetup.slotSetup[section][idx].info ~= nil then count = idx end
  end     
  
  local mid = math.floor(count / 2)

  local addCoRoutine = coroutine.create(
    function (button)
  
      for idx = 1, 40, 3 do
      
        local r = (baseRadius) + (button.sub[1]:GetWidth() / 2) +10 + (40 - idx)
        local basePos = button.main:GetPosition ()
        
        for idx = 1, count, 1 do          
          local diff = (idx - mid - 1) * 25
          if mid == count/2 then diff = diff + 12.5 end
          
          local thisRadius = button.sub[idx]:GetRadius()
          if r <= thisRadius then          
            button.sub[idx]:Position(basePos + diff, r)
          end
        end 
        
        if data.stopAnimation == false then        
          coroutine.yield(idx)
        else
          coroutine.yield(9999)
        end
      end
    end)
  
  local callBack = function ()
    for idx = 1, #nkRadialSetup.slotSetup[uiElements.buttons[buttonNo].section], 1 do
      uiElements.buttons[buttonNo].sub[idx]:SetVisible(false)
    end 
  end
  
  EnKai.coroutines.add ({ func = addCoRoutine, counter = 40, active = true, para1 = uiElements.buttons[buttonNo], callBack = callBack })

end

-------------------- STARTUP EVENTS --------------------

Command.Event.Attach(Event.Addon.Load.End, _fctMain, "nkRadial.Addon.Load.End")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, _fctSettingsHandler, "nkRadial.SavedVariables.Load.End")